<?
   // Anthill PGSQL connection class
   // $Id: pgsql.class.php,v 1.3 2002/07/24 21:35:21 vdanen Exp $
   // submitted by Phil Kenoyer

class db_Sql {
  // define connection info
  var $type   = "pgsql";
  var $host   = "";
  var $dbase  = "";
  var $user   = "";
  var $passwd = "";

  // some config items
  var $debug         = 0; // 1=debug messages, 0=off
  // How to handle errors:
  // 0 = ignore errors
  // 1 = halt with message
  // 2 = ignore error, but print a warning
  var $halt_on_error = 1;
  var $auto_free     = 0; // 1 = auto mysql_free_result()

  // result array and current row number
  var $record = array();
  var $row    = "";

  // current error number and text
  var $errno  = 0;
  var $error  = "";

  // link and query handles
  var $link_id  = 0;
  var $query_id = 0;

  // constructor
  function db_Sql($query = "") {
    $this->query($query);
  }

  // reporting
  function link_id() {
    return $this->link_id;
  }

  function query_id() {
    return $this->query_id;
  }

  // connection management
  function connect($dbase = "", $host = "", $user = "", $passwd = "") {
    // defaults
    if($dbase == "") {
      $dbase = $this->dbase;
    }
    if($host == "") {
      $host = $this->host;
    }
    if($user == "") {
      $user = $this->user;
    }
    if($passwd == "") {
      $passwd = $this->passwd;
    }

    // make connection
    if($this->link_id == 0) {
      if ($this->dbase) {
        $conn_str = "dbname=".$this->dbase;
      } else {
        $this->halt("No database name set.");
        return 0;
      }
      if ($this->host) $conn_str .= " host=".$this->host;
      if ($this->user) $conn_str .= " user=".$this->user;
      if ($this->passwd) $conn_str .= " password=".$this->passwd;

      $this->link_id = pg_pconnect($conn_str);
      if(!$this->link_id) {
        $this->halt("pconnect($host, $user, \$passwd) failed.");
        return 0;
      }
    }
    return $this->link_id;
  }

  // discard query results
  function free() {
    pg_freeresult($this->query_id);
    $this->query_id = 0;
  }

  // perform a query
  function query($query_string) {
    if($query_string == "") {
      // no empty queries allowed
      return 0;
    }
    if(!$this->connect()) {
      return 0;
    }
    if($this->query_id) {
      // this is a new query so get rid of old results
      $this->free();
    }
    // postgres doesn't like nulls where nulls don't belong (unlike mysql)
    // so we change "NULL" to "0"
    $query_string = ereg_replace("NULL","0",$query_string);
    if($this->debug) {
      printf("Debug: query = %s<br>\n", $query_string);
    }

    $this->query_id = pg_exec($this->link_id, $query_string);
    $this->row      = 0;
    $this->errno    = 0;
    $this->error    = pg_errormessage($this->link_id);
    if(!$this->query_id) {
      $this->halt("Invalid SQL: " . $query_string);
    }
    return $this->query_id;
  }

  function affected_rows() {
    return pg_affected_rows($this->link_id);
  }

  function num_rows() {
    return pg_num_rows($this->query_id);
  }

  function num_fields()	{
    return pg_num_fields($this->query_id);
  }

  function fetch_fieldname($num) {
    return pg_fieldname($this->query_id,$num);
  }

  function next_record() {
    if(!$this->query_id) {
      $this->halt("Next record called with no pending query.");
      return 0;
    }
    $this->record = pg_fetch_array($this->query_id);
    $this->row   += 1;
    $this->errno  = 0;
    $this->error  = pg_errormessage();

    $stat = is_array($this->record);
		
    if(!$stat && $this->auto_free) {
      $this->free();
    }
    return $stat;
  }

  // error handling
  function halt($msg) {
    $this->error = pg_errormessage($this->link_id);
    $this->errno = 0;
    if($this->halt_on_error == "0") {
      return;
    }
    $this->haltmsg($msg);
    if($this->halt_on_error != "2") {
      die("Session halted.");
    }
  }

  function haltmsg($msg) {
    printf("<b>Database error:</b> %s<br>\n", $msg);
    printf("<b>PGSQL Error</b>: %s (%s)<br>\n", $this->errno, $this->error);
  }
}
?>
