<?
   // Report Engine Functions
   // $Id: reportengine.php,v 1.1 2002/04/21 03:10:16 vdanen Exp $
   //
   // (C) 2002 by John Gatewood Ham
   //
   // This program is free software; you can redistribute it and/or modify
   // it under the terms of the GNU General Public License as published by
   // the Free Software Foundation; either version 2 of the License, or
   // (at your option) any later version.
   //
   // This program is distributed in the hope that it will be useful,
   // but WITHOUT ANY WARRANTY; without even the implied warranty of
   // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   // GNU General Public License for more details.
   //
   // You should have received a copy of the GNU General Public License
   // along with this program; if not, write to the Free Software
   // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
   //

   //
   // This will convert a date from MYSQL into a date suitable
   // for printing.  This version has hard-coded format so that
   // changes in the site.inc.php file won't break report layouts.
   // Also, all times are forced to be displayed in GMT so you 
   // can compare reports across time zones easily.
   //
   function d_getTimeGMT($date) {
      $year  = substr($date,0,4);
      $mon   = substr($date,4,2);
      $day   = substr($date,6,2);
      $hour  = substr($date,8,2);
      $min   = substr($date,10,2);
      $sec   = substr($date,12,2);
      $gdate = gmdate("Y-m-d H:i T",mktime($hour,$min,$sec,$mon,$day,$year));
      return($gdate);
   }

   //
   // This initializes the report engine.  There are 7 parameters, and
   // all have default values.  The parameters are:
   // rows     Total number of rows on the output
   // columns  Total number of columns on the output
   // ff       1 means add a ctl-L after each page, 0 means do not do this
   // isdos    1 means newlines are ms-dos style, 0 means Unix style
   // ht       This is the report title to put into the header of the report
   // lm       This is a left margin to indent.  0 means do not indent.
   // rm       This is a right margin to indent.  0 means do not indent.
   //          (lm and rm units are columns)
   //
   // examples:
   //    report_init();   
   //       accepts all default values
   //    report_init(10, 80, 1, 1, "Testing 3", 0, 0);
   //       10 lines, 80 columns, use form feeds, create ms-dos newlines,
   //       the report title is "Testing 3", do not indent left or right
   //       margins.
   //    report_init(58, 132, 1, 0, "Testing 2", 5, 5);
   //       58 lines, 132 columns, use forfeeds, Unix newlines, 
   //       report title is "Testing 2", indent 5 columns left and right.
   //
   function report_init($rows = 58, $columns = 80, $ff = 1, $isdos = 0,$ht = "", $lm = 0, $rm = 0) {
     $pageoftext['magic'] = "magic";
     $GLOBALS['report_headertext'] = $ht;
     $GLOBALS['report_pageoftext'] = $pageoftext; // lines in buffered page
     $GLOBALS['report_rows'] = $rows - 4; // reserve 4 lines for header/footer
     $GLOBALS['report_columns'] = $columns;
     $GLOBALS['report_linecount'] = 0; // number of lines in buffered page
     $GLOBALS['report_pageno'] = 0; // current page number
     // following 3 items are for poisitioning items in the header/footer
     $GLOBALS['report_left'] = (int)floor($GLOBALS['report_columns'] / 3.0);
     $GLOBALS['report_right'] = $GLOBALS['report_left'];
     $GLOBALS['report_center'] = $GLOBALS['report_columns'] - $GLOBALS['report_left'] - $GLOBALS['report_right'];
     $GLOBALS['report_do_ctl_l'] = $ff;
     $GLOBALS['report_leftmargin'] = $lm;
     $GLOBALS['report_rightmargin'] = $rm;
     $GLOBALS['report_linewidth'] = $columns - $lm - $rm; // columns remaining after margins
     $GLOBALS['report_keep'] = "off";    // are we in a keep block?
     $GLOBALS['report_keep_lines'] = 0;  // number of lines in keep
     $keeparea['magic'] = "magic";
     $GLOBALS['report_keep_area'] = $keeparea; // lines in keep
     
     // isdos 1 is do chr(13)chr(10), 0 is just chr(10)
     if ($isdos == 1) {
        $GLOBALS['report_newline'] = chr(13) . chr(10);
     } else {
        $GLOBALS['report_newline'] = chr(10);
     }
     return;
   }

   //
   // Begin a keep group.  A keep group is a group of lines
   // that we want to keep together on the same page.  If the
   // entire keep won't fit on the current page, a page eject
   // is done and the keep is started at the top of the next page.
   // returns 1 if we can start the keep, 0 otherwise
   //
   function report_start_keep() {
     if ($GLOBALS['report_keep'] == "off") {
        $GLOBALS['report_keep'] = "on";
        $GLOBALS['report_keep_lines'] = 0;
        return 1;
     }
     return 0;
   }

   //
   // End a keep group.  This transfers the lines in the keep
   // area to the buffered page, doing a page eject first if
   // necessary.
   //
   function report_end_keep() {
     if ($GLOBALS['report_keep'] == "on") {
        $ll = $GLOBALS['report_rows'] - $GLOBALS['report_linecount'];
        if ($GLOBALS['report_keep_lines'] > $ll) {
           report_page_eject();
        }
        $GLOBALS['report_keep'] = "off";
        for ($i=0;$i< $GLOBALS['report_keep_lines']; $i += 1) {
          report_que_line($GLOBALS['report_keep_area'][$i]);
        }
        $GLOBALS['report_keep_lines'] = 0;
     }
     return 0;
   }

   //
   // End a report.  This ends any open keep area, then
   // flushes any buffered lines with a page eject if necessary.
   //
   function report_finish() {
     if ($GLOBALS['report_keep'] == "on") {
        report_end_keep();
     }
     if ($GLOBALS['report_linecount'] > 0) {
        report_page_eject();
     }
     return;
   }

   //
   // This is an internal function to format one line of a header
   // or footer.
   //
   function report_do_header_line($l,$c,$r) {
     return (str_pad($l,$GLOBALS['report_left']," ", STR_PAD_RIGHT) .
             str_pad($c,$GLOBALS['report_center']," ", STR_PAD_BOTH) .
             str_pad($r,$GLOBALS['report_right']," ", STR_PAD_LEFT));
   }

   //
   // This is an internal function to print the header.
   //
   function report_do_header() {
     printf("%s%s%s",report_do_header_line(gmdate("D, d M Y"),$GLOBALS['report_headertext'],gmdate("H:i:s")),
        $GLOBALS['report_newline'],$GLOBALS['report_newline']);
     return;
   }

   //
   // This is an internal function to print the footer.
   //
   function report_do_footer() {
     $GLOBALS['report_pageno'] += 1;
     printf("%s%s%s",$GLOBALS['report_newline'],
            report_do_header_line("","Page " . $GLOBALS['report_pageno'], ""),
            $GLOBALS['report_newline']);
     if ($GLOBALS['report_do_ctl_l'] == 1) {
        print(chr(12));
     }
     return;
   }

   //
   // This function will perform a page eject, flushing any
   // buffered lines onto the current page before starting
   // a new page.
   //
   function report_page_eject() {
     report_do_header();

     for($i = 0; $i < $GLOBALS['report_linecount']; $i += 1) {
        printf("%s%s%s%s",str_pad("",$GLOBALS['report_leftmargin']," ",STR_PAD_RIGHT),
               $GLOBALS['report_pageoftext'][$i],
               str_pad("",$GLOBALS['report_rightmargin']," ",STR_PAD_RIGHT),
               $GLOBALS['report_newline']);
        $GLOBALS['report_pageoftext'][$i] = "XXX ($i)";
     }
     for(;$i < $GLOBALS['report_rows']; $i += 1) {
        printf("%s",$GLOBALS['report_newline']);
        $GLOBALS['report_pageoftext'][$i] = "YYY ($i)";
     }
     report_do_footer();
     $GLOBALS['report_linecount'] = 0;
     return;
   }

   //
   // This function will add the linedata to the current page buffer.
   // If it will not fit, a page eject is done and linedata is added
   // to the next page.  This routine also does primitive line wrapping
   // by breaking the input into report_linewidth sized chunks.  Use
   // report_flowtext for breaking at word boundaries.
   //
   function report_que_line($linedata) {
     $tstring = $linedata;
     $x = strlen($tstring);
     do {
       if ($GLOBALS['report_linecount']+1 > $GLOBALS['report_rows']) {
          report_page_eject();
       }
       if ($GLOBALS['report_keep'] == "on") {
            $GLOBALS['report_keep_lines'] += 1;
       } else {
            $GLOBALS['report_linecount'] += 1;
       }
       if ($x > $GLOBALS['report_linewidth']) {
         $ostring = substr($tstring,0,$GLOBALS['report_linewidth']);
         $tstring = substr($tstring,$GLOBALS['report_linewidth']);
         $x = strlen($tstring);
       } else {
         $ostring = $tstring;
         $tstring = "";
         $x = 0;
       }
       if ($GLOBALS['report_keep'] == "on") {
         $GLOBALS['report_keep_area'][$GLOBALS['report_keep_lines']-1] = $ostring;
       } else {
         $GLOBALS['report_pageoftext'][$GLOBALS['report_linecount']-1] = $ostring;
       }
     } while ($x > 0);
     return;
   }

   //
   // Internal function used by report_flowtext to compute where
   // word breaks can occur.
   // returns 0 to say no split required
   // returns place to split if a space is found to split on
   // if no space found to split on, split on report_linewidth value
   //
   function report_where2split($linedata,$lm = 0) {
     $tstring = $linedata;
     $x = strlen($tstring);
     if ($x > $GLOBALS['report_linewidth']-$lm) {
        $ostring = substr($tstring,0,$GLOBALS['report_linewidth']-$lm);
        $y = strrpos($ostring," ");
        if ($y > 0) {
           return($y);
        } else {
           return($GLOBALS['report_linewidth']-$lm);
        }
     }
     return(0);
   }

  //
  // This is like report_que_line(), but it does better line breaking
  // for long lines by breaking on spaces if possible. For example,
  // On a 10 column output, report_flowtext would do this:
  //
  // 0123456789
  // This was 
  // only a
  // test.
  //
  // but report_que_line() would do this:
  // 0123456789
  // This was o
  // nly a test
  // .
  //
  // Breaks are done on spaces (" ").
  //
  function report_flowtext($data,$lm = 0) {
     $outstring = $data;
     if ($lm > 0) {
        $tpad = str_pad("", $lm, " ", STR_PAD_LEFT);
     } else {
        $tpad = "";
     }
     do {
       $z = report_where2split($outstring,$lm);
       if ($z > 0) {
          report_que_line($tpad . substr($outstring,0,$z));
          $outstring = substr($outstring,$z+1);
          $z = report_where2split($outstring,$lm);
       }
     } while ($z > 0);
     if (strlen($outstring) > 0) {
        report_que_line($tpad . $outstring);
     }
     return;
  }

 ?>
